/*
// A simple procedural implementation of some numerical
// integration rules.
//
*/

#include "intrule.h"

// definition of the prototype of the trapezoidal integration rule
valueType
trap( functionType fun,
      valueType    a,
      valueType    b,
      indexType    N ) {
  valueType h = (b-a)/N ;
  valueType res = h*((*fun)(a) + fun(b))/2 ;
  //                    ^         ^
  //                    |         + a shortcut or syntactic glue from C
  //                    + the "correct way"
  for ( indexType i = 1 ; i < N ; ++i ) {
    valueType xi = a + h*i ;
    res += h*fun(xi) ;
  }
  return res ;
}

// definition of the prototype of the midpoint integration rule
valueType
midpoint( functionType fun,
          valueType    a,
          valueType    b,
          indexType    N ) {
  valueType h   = (b-a)/N ;
  valueType res = 0 ;
  for ( indexType i = 0 ; i < N ; ++i ) {
    valueType xi_plus_half = a + h*(i+0.5) ;
    res += h*fun(xi_plus_half) ;
  }
  return res ;
}

// definition of the prototype of the Simpson integration rule
valueType
simpson( functionType fun,
         valueType    a,
         valueType    b,
         indexType    N ) { // N must be even!
  valueType h   = (b-a)/N ;
  valueType res = h*((*fun)(a) + fun(b))/3 ;
  for ( indexType i = 1 ; i < N ; i += 2 ) {
    valueType xi = a + h*i ;
    res += (4*h/3)*fun(xi) ;
  }
  for ( indexType i = 2 ; i < N-1 ; i += 2 ) {
    valueType xi = a + h*i ;
    res += (2*h/3)*fun(xi) ;
  }
  return res ;
}
