/*
// A simple object oriented implementation of some numerical
// integration rules.
//
*/

#include "intrule.h"

valueType
trap::operator () ( valueType a, valueType b, indexType N ) {
  valueType h = (b-a)/N ;
  valueType res = h*(fun(a) + fun(b))/2 ;
  for ( indexType i = 1 ; i < N ; ++i ) {
    valueType xi = a + h*i ;
    res += h*fun(xi) ;
  }
  return res ;
}

valueType
midpoint::operator () ( valueType a, valueType b, indexType N ) {
  valueType h   = (b-a)/N ;
  valueType res = 0 ;
  for ( indexType i = 0 ; i < N ; ++i ) {
    valueType xi_plus_half = a + h*(i+0.5) ;
    res += h*fun(xi_plus_half) ;
  }
  return res ;
}

valueType
simpson::operator () ( valueType a, valueType b, indexType N ) {
  valueType h   = (b-a)/N ;
  valueType res = h*((*fun)(a) + fun(b))/3 ;
  for ( indexType i = 1 ; i < N ; i += 2 ) {
    valueType xi = a + h*i ;
    res += (4*h/3)*fun(xi) ;
  }
  for ( indexType i = 2 ; i < N-1 ; i += 2 ) {
    valueType xi = a + h*i ;
    res += (2*h/3)*fun(xi) ;
  }
  return res ;
}
