/*
// A simple object oriented implementation of some numerical
// integration rules.
//
*/

#include <iostream>

typedef double valueType ;
typedef int    indexType ;

typedef valueType (*functionType) ( valueType x ) ;

class numericalIntegrator {
protected:
  functionType fun ;
public:
  // default initialilizer
  numericalIntegrator()
  : fun(0)
  {} 

  numericalIntegrator(functionType ffun)
  : fun(ffun)
  {}
  
  virtual
  ~numericalIntegrator()
  {}
  
  // set up a funtion to integrate
  void
  setup( functionType ffun ) {
    fun = ffun ;
  }

  //
  //  To integrate a function if INT is of type numericalIntegrator
  //  INT . setup(fun);
  //  res = INT(a,b,N) ;
  // declaring the operator PURE VIRTUAL
  virtual
  valueType
  operator () ( valueType a, valueType b, indexType N ) const = 0 ;

} ;

class trap : public numericalIntegrator {
public:
  // default initialilizer
  trap()
  : numericalIntegrator()
  {} 

  trap(functionType ffun )
  : numericalIntegrator(ffun)
  {}

  virtual
  ~trap()
  {}

  //
  //  To integrate a function if INT is of type numericalIntegrator
  //  INT . setup(fun);
  //  res = INT(a,b,N) ;
  //
  virtual
  valueType
  operator () ( valueType a, valueType b, indexType N ) const ;

} ;

class midpoint : public numericalIntegrator {
public:
  // default initialilizer
  midpoint()
  : numericalIntegrator()
  {} 

  midpoint(functionType ffun )
  : numericalIntegrator(ffun)
  {}
  
  virtual
  ~midpoint()
  {}

  //
  //  To integrate a function if INT is of type numericalIntegrator
  //  INT . setup(fun);
  //  res = INT(a,b,N) ;
  //
  virtual
  valueType
  operator () ( valueType a, valueType b, indexType N ) const ;

} ;

class simpson : public numericalIntegrator {
public:
  // default initialilizer
  simpson()
  : numericalIntegrator()
  {} 

  simpson(functionType ffun )
  : numericalIntegrator(ffun)
  {}
  
  virtual
  ~simpson()
  {}

  //
  //  To integrate a function if INT is of type numericalIntegrator
  //  INT . setup(fun);
  //  res = INT(a,b,N) ;
  //
  virtual
  valueType
  operator () ( valueType a, valueType b, indexType N ) const ;

} ;
