/*
// A simple object oriented implementation of some numerical
// integration rules.
//
*/

#include "intrule.h"
#include <cmath>
#include <iostream>

using namespace std ;

static
valueType
testFun( valueType x ) {
  return x*exp(-x) ;
}

int
main() {

  numericalIntegrator * tInstance = new trap( testFun ) ;
  numericalIntegrator * mInstance = new midpoint( testFun ) ;
  numericalIntegrator * sInstance = new simpson( testFun ) ;

  valueType a = 0  ;
  valueType b = 10 ;

  for ( indexType N = 10 ; N <= 1000 ; N *= 2 ) {
  
    valueType t = (*tInstance)( a, b, N ) ; // call trapeziodal integrator
    valueType m = (*mInstance)( a, b, N ) ; // call midpoint integrator
    valueType s = (*sInstance)( a, b, N ) ; // call Simpson integrator
    
    // print the result
    cout << "Integration with " << N << " intervals\n"
         << "Trapezoidal rule " << t << "\n"
         << "Midpoint rule    " << m << "\n"
         << "Simspon rule     " << s << "\n" ;
  }

  delete tInstance ;
  delete mInstance ;
  delete sInstance ;

  cout << "All done!\n" ;

  return 0;
}
